!    Genealogical Evolution Model (GEM) is an efficient logical model used to track dynamic evolution of mesoscale eddies in the ocean. 
! It can distinguish different dynamic processes (e.g., merging and splitting) within a dynamic evolution pattern, which is difficult 
! to accomplish using other tracking methods. To this end, GEM first uses a two-dimensional (2-D) vector rather than a scalar to measure 
! the similarity between eddies, which effectively solves the missing eddy problem (temporally lost eddy in tracking). Second, 
! GEM uses both parents and children in tracking, and the dynamic processes are described as birth and death of different generations. 
! Additionally, a look-ahead approach with selection rules effectively simplifies computation and recording. All of the computational 
! steps are linear and do not include iteration. Given the pixel number of the target region L, the maximum number of eddies M, the 
! look-ahead time steps N, and the total number of time steps T, the total computation complexity is O (LM(N+1)T). The tracking of each 
! eddy is very smooth because we require that the snapshots of each eddy on adjacent days overlap one another.
!    
!    The codes can be freely used for study but not for commercial purpose.
!
!    The project is supported by the National Foundation of Natural Science (No. 41376017) of China.
!    
!                      Correponding to : Liang Sun, email: sunl@ustc.edu.cn; sunl@ustc.edu
!   School of Earth and Space Sciences, University of Science and Technology of China, 230026, Hefei, China

! MEI ģͲ1 Min_gridС
! MEI ģͲ2 Min_ampС
! 5ģͲMP_SSH_CV_MINMP_SSH_CV_MAXMP_AMP_CV_MINMP_Grid_Min,MP_GV_CV_MIN
!    
! GEM ģͲ1 Min_ratioغϱ
! GEM ģͲ2 MP_MAX_DAY=N+2׷ٶ=+һ+Look-aheadN
! 
! GEM вҪǳҪĲ

module constants
	implicit none

! ģͲ----MeiеĺѡMonoExtrId_MarkEddyRegionӳ
!  integer,parameter::MP_GRID_CV_MIN=121        ! OFESģͲеС>=121
!  real,parameter::MP_AMP_SSH_MIN=1.E-1         ! OFESģͲеСSSH
  integer,parameter::MP_GRID_CV_MIN=16           ! SLAģͲеС>=9
  real,parameter::MP_AMP_SSH_MIN=3.E-2          ! SLAģͲеСSSH
  real,parameter::MP_AMP_GV_MIN=3.E-2           ! ģͲеСGV
!  real,parameter::MP_AMP_OW_MIN=1.E-11         !ģͲеСOW
  real::MP_AMP_OW_MIN=1.E-11         !ģͲеСOW
  
!  real::MP_OW_CV_MIN=-1.D-12                   !ģͲOWжеϵͳOWͳƺ
  real,parameter::MP_OW_CV_MIN=0.D-12           !ģͲOWже
  real,parameter::MP_OW_CV_MAX=1.D0             !ģͲOWже
  real,parameter::MP_GV_CV_MIN=-1.D-3           !ģͲGV(Rossby number)же
  real,parameter::MP_GV_CV_MAX=1.D-3            !ģͲGV(Rossby number)же
  real,parameter::MP_SSH_CV_ROB=1.D0            !ģͲ, SSH߽жǷ
!  real,parameter::MP_SSH_CV_MIN=-3.E-1         !OFESģͲжSSH
!  real,parameter::MP_SSH_CV_MAX=3.E-1          !OFESģͲжSSH
  real,parameter::MP_SSH_CV_MIN=-3.E-2          !SLAģͲжSSH
  real,parameter::MP_SSH_CV_MAX=3.E-2           !SLAģͲжSSH
  integer,parameter::Max_Equator=401    !вϽ磨תЧ      
  integer,parameter::Min_Equator=321    !в½磨תЧ
  integer,parameter::MAX_Road=1000      !в½·;ҲͨĿ
  integer,parameter::Flag_mark=1        !в
  integer,parameter::Fake_MEI=-1        !вMEI
  
!  real,parameter::MP_GV_CV_MIN=0.5/MP_Eddylif_CV_MIN !ģͲСλУRossby̫СȡΪ
!  ҪͬʱSSHGVΪеĺѡ򣬼MonoExtrId_MarkEddyRegion 
! Link,look-ahead=MP_MAX_DAY-2
  
! ģͲ----VortexȽʺò
    real,parameter::MP_Rd_CV_MIN=0.6          !ģͲҪغ֮, Pi/4*D*D
    real,parameter::MP_Rc_CV_MIN1=1.0/real(MP_GRID_CV_MIN)       !ģͲVorغϰٷֱ
    real,parameter::MP_Rc_CV_MIN2=2.0/3.0      !ģͲVorغϰٷֱ:OWʱ1/3SSHʱ2/3
!    integer,parameter::MP_MAX_DAY=3            !OFESģͲLink,look-ahead=MP_MAX_DAY-2ļҲ׷+1Vortex/Eddy
    integer,parameter::MP_MAX_DAY=4            !SLAģͲLink,look-ahead=MP_MAX_DAY-2ļҲ׷+1Vortex/Eddy
! ϵͳԴMEI.f90Ҫ
	integer,parameter::MAX_LON=1440   ! ϵͳ1440
	integer,parameter::MAX_LAT=720    ! ϵͳ720
	integer,parameter::MAX_LON_R=1440   ! SLAϵͳ롢ʱҪǾֲ򣨲2*INI_LON+MAX_LON_R<=MAX_LON
	integer,parameter::MAX_LAT_R=720    ! SLAϵͳ롢ʱҪ    
!	integer,parameter::MAX_LON_R=600    ! OFESϵͳ롢ʱҪǾֲ򣨲2*INI_LON+MAX_LON_R<=MAX_LON
!	integer,parameter::MAX_LAT_R=560    ! OFESϵͳ롢ʱҪ    
    
	real,parameter::Deft_SSH=-990     ! ϵͳ
    real,parameter::Pi=3.1415926535   ! Pi
!    real,parameter::Start_LON=Pi/180.0*0.0    !OFESʼľ,OFESݲʹ
!    real,parameter::Start_LAT=Pi/180.0*5.0    !OFESʼγ,OFESݲʹ
!    real,parameter::d_LON=Pi/1800             !OFESSLA0.25㣬OFES0.1 
!    real,parameter::d_LAT=Pi/1800             !OFESSLA0.25㣬OFES0.1 
    real,parameter::Start_LON=0               !SLAʼľ,ݲʹ
    real,parameter::Start_LAT=-Pi/2.0         !SLAʼγ,ݲʹ
    real,parameter::d_LON=Pi/720              !SLASLA0.25㣬OFES0.1 
    real,parameter::d_LAT=Pi/720              !SLASLA0.25㣬OFES0.1 
    integer,parameter::MP_Bran_CV_MAX=3        !ģͲVorϲѸȡֵͬԵϵͳ
	integer,parameter::MAX_Extr=1500       ! вÿͼMEI/ExtremeSSHֵȽ٣OWǳࣩ
	integer,parameter::MAX_Vors=1200       !вÿͼVor/EddyMAX_Vors<MAX_Extr,MEI/Vor/Eddy
    integer,parameter::MP_MAX_LON_L1=25   ! вֲVorȽΧȸ
    integer,parameter::MP_MAX_LAT_L1=25   ! вֲVorȽΧγȸ
    integer,parameter::MP_MAX_LON_L2=51   ! вֲVorȽΧȸ
    integer,parameter::MP_MAX_LAT_L2=51   ! вֲVorȽΧγȸ
    integer,parameter::MP_MAX_LON_L3=81   ! вֲVorȽΧȸ
    integer,parameter::MP_MAX_LAT_L3=81   ! вֲVorȽΧγȸ
!	integer,parameter::INI_LON=MP_MAX_LON_L3/2+1          ! OFES0MP_MAX_LON_L3/2+1   ! ϵͳʼλ
!	integer,parameter::INI_LAT=0          ! OFESϵͳʼλ
	integer,parameter::INI_LON=0          ! SLA0MP_MAX_LON_L3/2+1   ! ϵͳʼλ
	integer,parameter::INI_LAT=0          ! SLAϵͳʼλ
	integer,parameter::MAX_Cov=20         ! вֲVorԱȿܳVor
	integer,parameter::Min_Vorgrid=9,MAX_Vorgrid=1000  !вڱȽϵVor
!	real,parameter::Min_ratio=2.0/3.0 !Vorغϰٷֱ
!    real,parameter::Min_Covarea=9     !Vorغϸȡ˲2010125

! Eddy
    integer,parameter::MP_Eddylif_CV_MIN=15      !ģͲСEddy life
	integer,parameter::MAX_Eddy=1000000 !вҵEddyп2-3
    integer,parameter::INI_Eddy=1000000  !вEddyʼıţп2-3
	integer,parameter::MAX_Edds=1500  !вÿͼVor/EddyMAX_Edds>=MAX_VorsMAX_Edds<=MAX_Extr
	integer,parameter::MAX_XEdds=200  !вÿͼгEddyÿΪMax_XLife
	integer,parameter::MAX_LIFE=400   !вͨEddyڵֵ
	integer,parameter::MAX_XLIFE=2200 !вEddyڵֵ
    integer,parameter::MAX_CoList=200 !вEddyĺϲ¼
    integer,parameter::MAX_SpList=200 !вEddyķѼ¼
	integer,parameter::Max_Gen=1      !вEddy¼+1
	integer,parameter::MAX_EddBran=9  !вÿϲĿĿΪ2*MP_Bran_CV_MAX
    integer,parameter::MAX_EddProp=5  !вϲѼ¼Ŀ
	integer,parameter::MAX_EddDixi=3  !вһΪףΪͬʱEddyѸMAX_EddDixi-1ӡ
    integer,parameter::MAX_Pare=MAX_EddBran     !вุ
    integer,parameter::MAX_Chil=MAX_EddBran     !вӱ
	integer,parameter::MAX_EddErzi=50 !вΪͬʱEddyѸMAX_EddErziӡ
	integer,parameter::Min_dis=5      !вСƶ
    integer,parameter::Fake_Eddy=-1   !вEddyı

!	integer,parameter::Deft_Eddy_End=-10

	character(20)::input1,input2,output1,output2
    character(100)::topofile,intxt
	character(80)::slapath,vorpath,eddypath
	character(100)::infile,outfile
	integer,parameter::fin=10,fout=20 !ļ豸
	integer::io_stat                  !ȡļ
	character(100)::inipara='E:\sunl\WORK\Eddytrack\inipara.txt'	!ļļ
	character(10)::inlist(10000)    !ļ
	integer::Num_files              !ļ
	integer::iconmin,iconmax        !ļӿʼͽ,inlist(iconmin)inlist(iconmax)

end module

module vortypes
use constants
implicit none

type Vorprop
!VorĲVor׷޹  
!ϢΪEddy brief reportsƷ'*-Ebr.dat'
!ȫΪEddy full recordsƷһ'*-Efr.dat'Ʒ
	integer::flag	  !Vor, 0=,1=Taylor,2=Rankine
	real::ssh         !VorļֵsshС
    real::amp         !VorʵVorԡ
	integer::grid     !Vor(ռĿ)
	real::area        !Vor()
    real::locx,locy   !Vorλ,ҲǼֵλie,je	
	real::radil,radis !VorԲ᳤,radil=radis=radiiΪԲ
	real::angle       !VorˮƽתǶ,0-2Pi.
end type

type Vorbr
!VorĲVorʶ׷޹  
!ϢΪVorticity brief reportsƷ'*-Vbr.dat'
!    integer::ID_Eddy    !VorӦEddyId,ڷѯС
    integer::locx,locy  !Vorλ,ҲǼֵλie,je	
	integer::grid     !Vor(ռĿ)
    real::amp         !VorʵVorԡ
 	real::ssh         !VorļֵsshС
end type

! MeiеVor¼'*-Vbr.dat'Ϊ: amp,locx,locy,grid
! Eddyеļ¼'Id_Eddy-Ebr.dat'ҪΪ time,amp,locx,locy,grid
! Table Meivor, ¼Mei-->VorŵıMei㷨ʱɡ
! VorrelaVoreddyʹãMeieddyMei-->Eddy

type Vorrela  !"*-Vorr.dat'
!Vor֮ϵĲVorĲ޹
!VorԹϵʱɣEddyʱֻΪƷ
!ʼtree_for/bakеVorΪMeiţ޸ΪVor
  integer::File_for,ntree_for            !ǰʱ(ļžֵ)Լ.
  integer::File_bak,ntree_bak            !̵ʱ(ļžֵ)Լ.
  integer::tree_for(MP_Bran_CV_MAX,2)        !̣ΪMP_Bran_CV_MAX
  integer::tree_bak(MP_Bran_CV_MAX,2)        !ǰMP_Bran_CV_MAXϲ
!  integer(4)::ID_Eddy    !EddyţTrackжнеıţ
! tree_for(*,1)ΪVorţ
! tree_for(*,2)ΪVorĹϵ:0ǰ,1ǰ,2ǰ,3ǰ
! Vor1%tree_for%tree_bak֮һȻָVor1˫ӡ 
end type

type Voreddy
!Vortex/Eddy֮ϵĲ
  integer::ID_Eddy    !EddyţTrackжнеıţ
  integer::ID_Track      !Eddy еı(,)
  integer::ID_Bran       !Trackеı(,)
end type

type VorFami
!Vortex/Eddy֮ϵĲ
  integer(4)::ID_Eddy    !EddyţTrackжнеıţ
  integer::ID_Track      !Eddy еı(,)
end type


type Vortex1        !VorǰTypeeddy
!  type(Vorprop)::vorp
  type(Vorbr)::vorp
  type(Vorrela)::vorr
  type(Voreddy)::vores(Max_Gen)   !Լ(Max_Gen==1)(Max_Gen==2)Eddy,Typeeddyʹã
!  integer::vorlink(2) !idv1,ifile1,idv2,ifile2,Typeeddyʹã
!  type(Voreddy)::vore              !ԼEddy,Typevorsʹã
!  integer::Vore(3,2) !Id1,Id2,It1,It2,dizi,erzivoresӦ  
end type

type Vortex2        !VorǰTypeVorsнVor-likeʹ
!  type(Vorprop)::vorp
  integer::vorid
  type(Vorbr)::vorp
  type(Vorrela)::vorr
!  type(Voreddy)::vore
end type

type Eddy
!  integer(4)::ID_Eddy    !Eddy id / Universal one for whole data.
  type(Voreddy)::vores(Max_Gen)     !ԼEddy,Ǽ̳EddyԵVortex%vore̳,1:ԣ2һ
!  integer::Num_Ances           !жٸȣƺˣParent(:)          
  integer::Id_Ini,Id_End,Life  !Eddyʼʱļ, Eddy Life
  integer::exten               !Max_Life Ϣλ
  integer::ivor                !Id_file,ivor Ϊһҵ
  integer::nextday
  integer::istart,itermi       !ĿǰûʵʹãҪ޶пʼͣ-1:쳣0=1ѣ2ϲ 
  integer::CoList(1+MAX_EddBran*MAX_EddProp,MAX_CoList),Num_Colist !ϲеʷ(:,1)Id_file(:,2)Id_Eddy, (:,3): Type
  integer::SpList(1+MAX_EddBran*MAX_EddProp,MAX_SpList),Num_Splist !ѵкʷ
  integer::Parent(1+2*MAX_EddBran),Num_Pare !
  integer::Child(1+2*MAX_EddBran),Num_Chil  !ӱ

  integer::Tracks(MAX_EddDixi,MAX_LIFE)       !Eddyʷ(¼ΪVor)Max_life
  integer::Request(MAX_EddBran,4),Num_Req             !ֱӼ̳λõıҪÿ!
!  type(Vorrela)::vorr                   !EddyVor¼Vorԣжս᷽ʽ
  type(Vorbr)::vorps(MAX_LIFE)          !EddyҪ
end type

type Eddext
  integer::Tracks(MAX_EddDixi,MAX_XLIFE)    !Eddyʷ(¼ΪVor)Max_life
  type(Vorbr)::vorps(MAX_XLIFE)             !EddyҪ
end type

end module 